package com.vendor.burn.model.devices.burners;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.model.BurnContext;
import com.vendor.burn.model.ChipID;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.FlashType;
import com.vendor.burn.model.PartitionInfo;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.TransferWay;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IErasable;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.devices.IPartitionBurner;
import com.vendor.burn.model.devices.ITab;
import com.vendor.burn.model.devices.burners.AbstractBurner;
import com.vendor.burn.model.event.EventType;
import com.vendor.burn.model.event.FastbootEvent;
import com.vendor.burn.serial.workers.AbstractBurnerWorker;
import com.vendor.burn.serial.workers.PartitionBurnerWorker;
import com.vendor.burn.sparse.SparseFileAgent;
import com.vendor.burn.ui.burners.tabs.PartitionTab;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeListener;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;

public class PartitionBurner extends AbstractBurner implements IPartitionBurner, IErasable {
  static final String FASTBOOT = "fastboot";
  
  private File partitionFile;
  
  protected List<PartitionItem> children;
  
  private long lestLen = 0L;
  
  private final int IS_CANCEL = -10;
  
  private PartitionInfo info = new PartitionInfo();
  
  public PartitionBurner(BurnContext burnContext) {
    super(burnContext);
    PartitionItem fastboot = new PartitionItem();
    fastboot.setName("fastboot");
    getChildren().add(fastboot);
  }
  
  public void burn(IFastbootProgressMonitor fastbootMonitor) {
    if (fastbootMonitor == null)
      return; 
    fastbootMonitor.getEvent().init(EventType.BURN);
    BurnContext context = getBurnContext();
    PartitionBurnerWorker worker = getBurnerWorker();
    worker.setProgressMonitor(fastbootMonitor);
    if (fastbootMonitor != null) {
      FastbootEvent event = fastbootMonitor.getEvent();
      Object object = new Object(this, "Burn By Partition", fastbootMonitor, worker, context, event);
      showProgressDialog((Job)object);
      object.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
      object.schedule();
    } 
  }
  
  private File getParentFile() {
    if (this.children == null || this.children.isEmpty())
      return null; 
    for (PartitionItem partition : this.children) {
      if (partition.isChecked() && 
        partition.getFile() != null)
        return partition.getFile().getParentFile(); 
    } 
    return null;
  }
  
  public IStatus checkBeforeBurn() {
    IStatus check = super.checkBeforeBurn();
    if (!isOK(check))
      return check; 
    check = checkTable(EventType.BURN);
    if (!isOK(check))
      return check; 
    if (!isOK(checkHasSelectNullFile()))
      return Status.CANCEL_STATUS; 
    return null;
  }
  
  public IStatus checkBeforeErase() {
    return checkTable(EventType.ERASE);
  }
  
  public IStatus checkBeforeMakePro() {
    IStatus check = checkTable(EventType.CREATE_IMAGE);
    if (!isOK(check))
      return check; 
    return null;
  }
  
  public boolean isOpenSerailPort(TransferWay transferWay) {
    if (transferWay == null)
      return false; 
    return (transferWay != TransferWay.Usb2I2c && transferWay != TransferWay.I2CBootrom && transferWay != TransferWay.USBBootrom && transferWay != TransferWay.USBNonNude && transferWay != TransferWay.JTAG && transferWay != TransferWay.JTAGAndNet && transferWay != TransferWay.JTAGAndSerial);
  }
  
  public void erase(IFastbootProgressMonitor fastbootMonitor) {
    if (fastbootMonitor == null)
      return; 
    fastbootMonitor.getEvent().init(EventType.ERASE);
    BurnContext context = getBurnContext();
    PartitionBurnerWorker worker = getBurnerWorker();
    worker.setProgressMonitor(fastbootMonitor);
    Object object = new Object(this, "Erase By Partition", fastbootMonitor, context, worker);
    showProgressDialog((Job)object);
    object.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    object.schedule();
  }
  
  public void eraseAll(IFastbootProgressMonitor fastbootMonitor) {
    if (fastbootMonitor == null)
      return; 
    fastbootMonitor.getEvent().init(EventType.ERASE);
    BurnContext context = getBurnContext();
    PartitionBurnerWorker worker = getBurnerWorker();
    worker.setProgressMonitor(fastbootMonitor);
    Object object = new Object(this, "Erase By Partition", fastbootMonitor, context, worker);
    showProgressDialog((Job)object);
    object.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    object.schedule();
  }
  
  protected IStatus checkTable(EventType event) {
    List<PartitionItem> items = getChildren();
    int size = items.size();
    int index = 0;
    int selection = 0;
    for (index = 0; index < size; index++) {
      if (((PartitionItem)items.get(index)).isChecked()) {
        selection++;
        break;
      } 
    } 
    if (0 == selection)
      return error(Messages.PartitionBurner_msg_no_burning_partition); 
    IStatus check = checkFastbootPart(items.get(0), event);
    if (!isOK(check)) {
      highLight(items);
      return check;
    } 
    check = checkOtherPart(items, event);
    if (!isOK(check))
      return check; 
    return null;
  }
  
  private void dealWithException(IFastbootProgressMonitor fastbootMonitor, IProgressMonitor monitor, Throwable e) {
    monitor.setCanceled(true);
    String messages = e.getMessage();
    if (fastbootMonitor != null && fastbootMonitor.getEvent() != null)
      if ("Java heap space".equalsIgnoreCase(messages)) {
        fastbootMonitor.getEvent().setMessage(Messages.MESSAGE_JAVAHEASPACE_ERROR);
      } else {
        fastbootMonitor.getEvent().setMessage(e.getMessage());
      }  
  }
  
  private void highLight(List<PartitionItem> items) {
    if (!ToolUtils.isCmdLine())
      getBurnContext().getUIAdapter().highLight(items.get(0)); 
  }
  
  protected IStatus checkOtherPart(List<PartitionItem> parts, EventType event) {
    int partSize = parts.size();
    int partIndex = 0;
    IStatus result = Status.OK_STATUS;
    PartitionItem fastboot = parts.get(partIndex++);
    long spiTotalLength = 0L;
    long nandTotalLength = 0L;
    if (fastboot.isChecked() && "fastboot".equalsIgnoreCase(fastboot.getName())) {
      File file = fastboot.getFile();
      if (event != EventType.ERASE) {
        if (file == null) {
          highLight(fastboot);
          return error(
              NLS.bind(Messages.AbstractBurner_msg_file_path_does_not_select_in_row_, Integer.valueOf(partIndex)));
        } 
        if (!file.exists()) {
          highLight(fastboot);
          return error(
              NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, Integer.valueOf(partIndex + 1)));
        } 
      } 
      if (FlashType.spi == fastboot.getFlashType()) {
        spiTotalLength = fastboot.getLength();
      } else if (FlashType.nand == fastboot.getFlashType()) {
        nandTotalLength = fastboot.getLength();
      } 
    } 
    FileSystem[] fileSystems = FileSystem.getFileSystems(this);
    long lastAddress = 0L;
    for (partIndex = 1; partIndex < partSize; partIndex++) {
      PartitionItem part = parts.get(partIndex);
      FlashType flashType = part.getFlashType();
      FileSystem fileSystem = part.getFileSystem();
      if (!part.isChecked())
        continue; 
      result = checkAddressOverlap(part, flashType, spiTotalLength, nandTotalLength, partIndex);
      if (result != Status.OK_STATUS)
        return result; 
      result = checkFlashAndFileSystem(partIndex, part, flashType, fileSystem, fileSystems);
      if (result != Status.OK_STATUS)
        return result; 
      result = checkStartAddressAndLen(part, partSize, partIndex);
      if (result != Status.OK_STATUS) {
        highLight(part);
        return result;
      } 
      long start = part.getStart();
      if (part.getLength() == 0L) {
        highLight(part);
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_nonzero_in_row_, 
              Integer.valueOf(partIndex + 1)));
      } 
      if (start < lastAddress) {
        highLight(part);
        return error(NLS.bind(Messages.AbstractBurner_messgae_the_partition_address_must_be_greater_than_the_used_size_in_row_, 
              
              Integer.valueOf(partIndex + 1)));
      } 
      lastAddress = start + ((part.getLength() == -1L) ? 0L : part.getLength());
      if (EventType.ERASE == event)
        continue; 
      File file = part.getFile();
      if (EventType.CREATE_IMAGE != event) {
        if (file == null)
          continue; 
      } else {
        if (file == null) {
          highLight(part);
          return error(NLS.bind(Messages.AbstractBurner_msg_file_path_does_not_select_in_row_, 
                Integer.valueOf(partIndex + 1)));
        } 
        if (!file.exists()) {
          highLight(part);
          return error(NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, Integer.valueOf(partIndex + 1)));
        } 
      } 
      result = checkStartAndLen(part, partIndex, partSize);
      if (result != Status.OK_STATUS)
        return result; 
      result = checkYaffsFile(partIndex, part, fileSystem);
      if (result != Status.OK_STATUS)
        return result; 
      result = checkNetFileNameLen(partIndex, file);
      if (result != Status.OK_STATUS)
        return result; 
      continue;
    } 
    return Status.OK_STATUS;
  }
  
  protected IStatus checkStartAddressAndLen(PartitionItem part, int partSize, int partIndex) {
    long start = part.getStart();
    long length = part.getLength();
    if (partIndex != partSize - 1) {
      if (0L == length)
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_nonzero_in_row_, 
              Integer.valueOf(partIndex + 1))); 
      if (length < 0L)
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_Less_than_zero_in_row_, 
              Integer.valueOf(partIndex + 1))); 
      if (start < 0L)
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_Less_than_zero_in_row_, 
              Integer.valueOf(partIndex + 1))); 
    } else {
      if (length < -1L)
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_Less_than_zero_in_row_, 
              Integer.valueOf(partIndex + 1))); 
      if (start < 0L)
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_Less_than_zero_in_row_, 
              Integer.valueOf(partIndex + 1))); 
    } 
    return Status.OK_STATUS;
  }
  
  protected IStatus checkStartAndLen(PartitionItem part, int index, long size) {
    long start = part.getStart();
    long length = part.getLength();
    File file = part.getFile();
    FlashType flashType = part.getFlashType();
    if (file != null && (!file.exists() || file.isDirectory()))
      return error(NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, Integer.valueOf(index + 1))); 
    if (file != null) {
      long fileSize = file.length();
      if (fileSize == 0L) {
        highLight(part);
        return error(NLS.bind(Messages.AbstractBurner_msg_the_file_size_must_be_nonzero_in_row_, Integer.valueOf(index + 1)));
      } 
      if (index != size - 1L) {
        if (length < fileSize) {
          highLight(part);
          return error(NLS.bind(Messages.PartitionBurner_msg_invalid_partition_size_in_row_, part
                .getFile().getName()));
        } 
      } else if (length != -1L) {
        if (length < fileSize) {
          highLight(part);
          return error(NLS.bind(Messages.PartitionBurner_msg_invalid_partition_size_in_row_, Integer.valueOf(index + 1)));
        } 
      } 
    } 
    int alignLen = 512;
    if (flashType == FlashType.ufs)
      alignLen = 4096; 
    if (start % alignLen != 0L) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_msg_partition_Start_should_be_with_unit_KorM_or_NK_times, 
            Integer.valueOf(alignLen), Integer.valueOf(index + 1)));
    } 
    if (index != size - 1L && 
      length % alignLen != 0L) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_msg_Length_must_be_NK_times_that_is_should_be_with_unit_K_or_M_in_row_, 
            
            Integer.valueOf(alignLen), Integer.valueOf(index + 1)));
    } 
    return Status.OK_STATUS;
  }
  
  private IStatus checkAddressOverlap(PartitionItem part, FlashType flashType, long spiTotalLength, long nandTotalLength, int index) {
    long start = part.getStart();
    if (FlashType.spi == flashType) {
      ChipID chipID = part.getChipID();
      if (0L == start && 0L != spiTotalLength && (
        chipID == null || chipID != ChipID.chip1)) {
        highLight(part);
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_nonzero_in_row_, 
              Integer.valueOf(index + 1)));
      } 
      if (0L != spiTotalLength && start < spiTotalLength && (
        chipID == null || chipID != ChipID.chip1)) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, Integer.valueOf(index + 1)));
      } 
    } else if (FlashType.nand == flashType) {
      if (0L == start && 0L != nandTotalLength) {
        highLight(part);
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_nonzero_in_row_, 
              Integer.valueOf(index + 1)));
      } 
      if (0L != nandTotalLength && start < nandTotalLength) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, Integer.valueOf(index + 1)));
      } 
    } else if (FlashType.eeprom != flashType) {
      highLight(part);
      return error(NLS.bind(Messages.AbstractBurner_msg_invalid_flash_type_in_row_, Integer.valueOf(index + 1)));
    } 
    return Status.OK_STATUS;
  }
  
  protected void highLight(PartitionItem part) {
    if (!ToolUtils.isCmdLine())
      getBurnContext().getUIAdapter().highLight(part); 
  }
  
  private IStatus checkFlashAndFileSystem(int index, PartitionItem part, FlashType flashType, FileSystem fileSystem, FileSystem[] fileSystems) {
    boolean validFileType = false;
    for (FileSystem req : fileSystems) {
      if (req == fileSystem)
        validFileType = true; 
    } 
    if (!validFileType) {
      highLight(part);
      return error(NLS.bind(Messages.AbstractBurner_msg_invalid_file_system_in_row_, Integer.valueOf(index + 1)));
    } 
    if (FlashType.spi == flashType && FileSystem.yaffs == fileSystem) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_msg_flash_SPI_and_file_system_Yaffs_can_not_be_selected_at_the_same_time_in_row_, 
            
            Integer.valueOf(index + 1)));
    } 
    if (FileSystem.ubiFs == fileSystem && FlashType.nand != flashType) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_UBI_only_support_Nand_flash, Integer.valueOf(index + 1)));
    } 
    if (SparseFileAgent.isSparse(fileSystem) && FlashType.emmc != flashType) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_EXT3_only_support_Emmc_flash, Integer.valueOf(index + 1)));
    } 
    return Status.OK_STATUS;
  }
  
  protected IStatus checkYaffsFile(int index, PartitionItem part, FileSystem fileSystem) {
    File file = part.getFile();
    if (file == null || !file.exists() || file.isDirectory())
      return Status.OK_STATUS; 
    boolean yaffsFile = false;
    try {
      yaffsFile = FileSystem.isYaffsFile((IBurner)this, file);
    } catch (IOException e) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_msg_failed_to_open_the_file_in_row_, Integer.valueOf(index + 1)));
    } 
    if (yaffsFile) {
      if (FileSystem.yaffs != fileSystem) {
        highLight(part);
        return error(NLS.bind(Messages.PartitionBurner_msg_file_is_yaffs_but_file_system_is_not_in_row_, 
              Integer.valueOf(index + 1)));
      } 
    } else if (FileSystem.none != fileSystem && FileSystem.ubiFs != fileSystem && (FileSystem.squashFs != fileSystem || fileSystem == FileSystem.f2fs) && FileSystem.ext3 != fileSystem && fileSystem != FileSystem.erofs && FileSystem.TAT32 != fileSystem && fileSystem != FileSystem.yaffs) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_msg_file_is_not_yaffs_but_file_system_is_yaffs_in_row_, 
            Integer.valueOf(index + 1)));
    } 
    return Status.OK_STATUS;
  }
  
  protected IStatus checkNetFileNameLen(int index, File file) {
    if (file == null)
      return Status.OK_STATUS; 
    try {
      if ((getBurnContext().getTransferWay() == TransferWay.Net || 
        getBurnContext().getTransferWay() == TransferWay.UsbTransNet || getBurnContext().getTransferWay() == TransferWay.JTAGAndNet) && (file
        .getName().getBytes(System.getProperty("file.encoding"))).length > 100)
        return error(NLS.bind(Messages.msg_File_name_is_too_long_please_use_a_short_name_instead, Integer.valueOf(index + 1))); 
    } catch (UnsupportedEncodingException unsupportedEncodingException) {}
    return Status.OK_STATUS;
  }
  
  protected IStatus checkFastbootPart(PartitionItem fastboot, EventType event) {
    if (!fastboot.isChecked())
      return null; 
    File file = fastboot.getFile();
    if (event == EventType.UPLOAD) {
      if (file == null) {
        highLight(fastboot);
        return error(NLS.bind(Messages.AbstractBurner_msg_file_path_does_not_select_in_row_, Integer.valueOf(1)));
      } 
      if (file != null && file.exists()) {
        boolean openConfirm = false;
        try {
          openConfirm = MessageDialog.openQuestion(
              Display.getDefault().getActiveShell(), Messages.FASTBOOT_CONFIRM_Title, 
              
              NLS.bind(Messages.PartitionComposite_msg_file_exists_already_whether_to_overwrite_it, file
                .getCanonicalPath()));
        } catch (IOException e) {
          return Status.CANCEL_STATUS;
        } 
        if (!openConfirm) {
          getBurnContext().getUIAdapter().highLight(fastboot);
          return error(Messages.AbstractBurner_msg_file_exists_please_check_file);
        } 
      } else {
        return null;
      } 
    } 
    FileSystem fileSystem = fastboot.getFileSystem();
    long start = fastboot.getStart();
    long length = fastboot.getLength();
    long fileSize = 0L;
    if (event != EventType.ERASE) {
      if (file == null) {
        highLight(fastboot);
        return error(NLS.bind(Messages.AbstractBurner_msg_file_path_does_not_select_in_row_, Integer.valueOf(1)));
      } 
      if (!file.exists()) {
        highLight(fastboot);
        return error(NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, Integer.valueOf(1)));
      } 
      fileSize = file.length();
      if (fileSize == 0L)
        return error(NLS.bind(Messages.AbstractBurner_msg_the_file_size_must_be_nonzero_in_row_, Integer.valueOf(1))); 
      try {
        boolean needCheckFilelen = ((getBurnContext().getTransferWay() == TransferWay.Net || getBurnContext().getTransferWay() == TransferWay.UsbTransNet || getBurnContext().getTransferWay() == TransferWay.JTAGAndNet) && (file.getName().getBytes(System.getProperty("file.encoding"))).length > 100);
        if (needCheckFilelen)
          return error(NLS.bind(Messages.msg_File_name_is_too_long_please_use_a_short_name_instead, Integer.valueOf(1))); 
      } catch (UnsupportedEncodingException unsupportedEncodingException) {}
    } 
    if (fileSystem != FileSystem.none)
      return error(NLS.bind(Messages.PartitionBurner_msg_invalid_fastboot_fileSystem, Integer.valueOf(1))); 
    if (start != 0L)
      return error(NLS.bind(Messages.PartitionBurner_msg_invalid_fastboot_start, Integer.valueOf(1))); 
    if (0L == length)
      return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_nonzero_in_row_, Integer.valueOf(1))); 
    if (length < fileSize)
      return error(NLS.bind(Messages.PartitionBurner_msg_invalid_fastboot_length, fastboot.getFile().getName())); 
    int alignLen = 1024;
    if (fastboot.getFlashType() == FlashType.ufs)
      alignLen = 4096; 
    if (length % alignLen != 0L)
      return error(NLS.bind(Messages.PartitionBurner_msg_Length_must_be_NK_times_that_is_should_be_with_unit_K_or_M_in_row_, 
            
            Integer.valueOf(alignLen), Integer.valueOf(1))); 
    return Status.OK_STATUS;
  }
  
  public List<PartitionItem> getChildren() {
    if (this.children == null)
      this.children = this.info.getChildren(); 
    return this.children;
  }
  
  public List<String> getSteps() {
    List<String> steps = new ArrayList<>();
    steps.add(Messages.PartitionBurner_Step_1);
    steps.add(Messages.PartitionBurner_Step_2);
    steps.add(Messages.PartitionBurner_Step_3);
    steps.add(Messages.PartitionBurner_Step_4);
    steps.add(Messages.PartitionBurner_Step_5);
    steps.add(Messages.PartitionBurner_Step_6);
    return steps;
  }
  
  public void remove(PartitionItem child) {
    getChildren().remove(child);
  }
  
  public void setChildren(List<PartitionItem> children) {
    this.children = children;
  }
  
  public void setTableFile(File tableFile) {
    setPartitionFile(tableFile);
  }
  
  public File getPartitionFile() {
    return this.partitionFile;
  }
  
  public void setPartitionFile(File partitionFile) {
    this.partitionFile = partitionFile;
  }
  
  public ITab getTab() {
    return (ITab)new PartitionTab();
  }
  
  protected PartitionBurnerWorker getBurnerWorker() {
    return new PartitionBurnerWorker((IBurner)this);
  }
  
  public void createImage(IFastbootProgressMonitor fastbootMonitor) {
    PartitionBurnerWorker worker = getBurnerWorker();
    worker.setProgressMonitor(fastbootMonitor);
    FileDialog d = new FileDialog(Display.getDefault().getActiveShell(), 8192);
    String open = d.open();
    if (open == null)
      return; 
    File file = new File(open);
    if (file.isDirectory()) {
      MessageDialog.openWarning(Display.getDefault().getActiveShell(), Messages.FASTBOOT_WARNING_Title, "Can not be a Directory!");
      return;
    } 
    Object object = new Object(this, "Making Image...", fastbootMonitor, worker);
    showProgressDialog((Job)object);
    object.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    object.schedule();
  }
  
  public IStatus checkCreateImage() {
    List<PartitionItem> children = getChildren();
    int size = children.size();
    if (children.isEmpty())
      return null; 
    long lastAddress = 0L;
    FlashType[] flashTypes = FlashType.getFlashTypes(this);
    FileSystem[] fileSystems = FileSystem.getFileSystems(this);
    for (int i = 0; i < size; i++) {
      PartitionItem partition = children.get(i);
      long start = partition.getStart();
      if (partition.isChecked()) {
        if (partition.getFlashType() == FlashType.spi) {
          highLight(partition);
          return error(NLS.bind(Messages.PartitionBurner_msg_invalid_flash_type, Integer.valueOf(i + 1)));
        } 
        boolean flag = (0 == i && start != 0L);
        if (flag) {
          highLight(partition);
          return error(Messages.PartitionBurner_msg_invalid_fastboot_start);
        } 
        if (0L == partition.getLength()) {
          highLight(partition);
          return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_nonzero_in_row_, Integer.valueOf(i + 1)));
        } 
        if (start < lastAddress) {
          highLight(partition);
          return error(NLS.bind(Messages.AbstractBurner_messgae_the_partition_address_must_be_greater_than_the_used_size_in_row_, 
                
                Integer.valueOf(i + 1)));
        } 
        boolean valid = false;
        for (FileSystem fileSystem : fileSystems) {
          if (fileSystem == partition.getFileSystem()) {
            valid = true;
            break;
          } 
        } 
        if (!valid) {
          highLight(partition);
          return error(NLS.bind(Messages.AbstractBurner_msg_invalid_file_system_in_row_, Integer.valueOf(i + 1)));
        } 
        valid = false;
        for (FlashType flashType : flashTypes) {
          if (flashType == partition.getFlashType()) {
            valid = true;
            break;
          } 
        } 
        if (!valid) {
          highLight(partition);
          return error(NLS.bind(Messages.AbstractBurner_msg_invalid_flash_type_in_row_, Integer.valueOf(i + 1)));
        } 
        lastAddress = start + ((partition.getLength() == -1L) ? 0L : partition.getLength());
        int alignLen = 512;
        if (partition.getFlashType() == FlashType.ufs)
          alignLen = 4096; 
        boolean partitionFlag = (partition.getLength() == -1L && partition.getLength() % alignLen != 0L && i != size - 1);
        if (partitionFlag) {
          highLight(partition);
          return error(NLS.bind(Messages.PartitionBurner_msg_Length_must_be_NK_times_that_is_should_be_with_unit_K_or_M_in_row_, 
                
                Integer.valueOf(alignLen), Integer.valueOf(i + 1)));
        } 
        if (partition.isChecked()) {
          File file = partition.getFile();
          if (file == null)
            return error(NLS.bind(Messages.AbstractBurner_msg_file_path_does_not_select_in_row_, Integer.valueOf(i + 1))); 
          if (start % alignLen != 0L) {
            highLight(partition);
            return error(NLS.bind(Messages.PartitionBurner_msg_partition_Start_should_be_with_unit_KorM_or_NK_times, 
                  Integer.valueOf(alignLen), 
                  Integer.valueOf(i + 1)));
          } 
          if (!file.exists())
            return error(NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, Integer.valueOf(i + 1))); 
          if (file.isDirectory()) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_the_file_can_not_be_a_directory, Integer.valueOf(i + 1)));
          } 
          FileInputStream fileInputStream = null;
          try {
            fileInputStream = new FileInputStream(file);
          } catch (Exception e) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_the_file_can_not_be_opened_in_row_, Integer.valueOf(i + 1)));
          } finally {
            if (fileInputStream != null)
              try {
                fileInputStream.close();
              } catch (Exception exception) {} 
          } 
          if (0L == file.length()) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_the_file_size_must_be_nonzero_in_row_, Integer.valueOf(i + 1)));
          } 
          if (partition.getLength() != -1L && partition.getLength() < file.length()) {
            highLight(partition);
            return error(NLS.bind(Messages.PartitionBurner_msg_invalid_partition_size_in_row_, partition));
          } 
        } 
      } 
    } 
    return null;
  }
  
  public boolean checkLastPartition(List<PartitionItem> parts, IFastbootProgressMonitor monitor) {
    long lastLen = getLastPartitionLength(parts);
    int size = parts.size();
    PartitionItem partition = parts.get(size - 1);
    long start = partition.getStart();
    if (partition.getLength() == -1L) {
      if (lastLen == -10L)
        return false; 
      if (lastLen % 1024L != 0L) {
        Display.getDefault().syncExec((Runnable)new Object(this));
        return false;
      } 
      if (0L == lastLen) {
        Display.getDefault().syncExec((Runnable)new Object(this));
        return false;
      } 
      if (start > lastLen) {
        Display.getDefault().syncExec((Runnable)new Object(this));
        return false;
      } 
      partition.setLength(lastLen);
      if (monitor != null)
        monitor.refreshUiPartitionTable(); 
    } 
    return true;
  }
  
  public long getLastPartitionLength(List<PartitionItem> parts) {
    int size = parts.size();
    PartitionItem partition = parts.get(size - 1);
    long partitionLength = partition.getLength();
    long start = partition.getStart();
    if (partitionLength == -1L)
      Display.getDefault().syncExec((Runnable)new Object(this, start)); 
    if (this.lestLen == 0L)
      return -10L; 
    return this.lestLen;
  }
  
  protected IStatus checkHasSelectNullFile() {
    boolean checkPartitionFileHasNull = false;
    StringBuilder nullFilePartNames = new StringBuilder();
    for (PartitionItem partitionItem : this.children) {
      if (partitionItem.isChecked()) {
        File file = partitionItem.getFile();
        if (file == null) {
          checkPartitionFileHasNull = true;
          String partitionName = "";
          if (partitionItem.getName() != null && partitionItem.getName().isEmpty()) {
            partitionName = " ";
          } else {
            partitionName = partitionItem.getName();
          } 
          nullFilePartNames.append(partitionName + ",");
        } 
      } 
    } 
    if (checkPartitionFileHasNull) {
      boolean questionResult = MessageDialog.openConfirm(Display.getCurrent().getActiveShell(), Messages.FASTBOOT_MessageDialog_Title, Messages.EmmcBurner_CheckHasSelectNullFile + "[" + Messages.EmmcBurner_CheckHasSelectNullFile + "]");
      if (!questionResult)
        return Status.CANCEL_STATUS; 
    } 
    return Status.OK_STATUS;
  }
  
  public PartitionInfo getPratitionInfo() {
    return this.info;
  }
  
  public void burn(IFastbootProgressMonitor monitor, String directory) {}
}
